
using System;
using System.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using Microsoft.Practices.EnterpriseLibrary.Security.Configuration;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using System.Collections.Generic;
using Microsoft.Practices.EnterpriseLibrary.Security;
namespace Kreeg.EnterpriseLibrary.Security.Database.Authorization
{
    /// <summary>
    /// Represents the configuration settings for the <see cref="DbRulesAuthorizationProvider"/>.
    /// </summary>
    //[XmlRoot("authorizationProvider", Namespace = SecuritySettings.)]
    public class DbRulesAuthorizationProviderData : AuthorizationProviderData
    {
        private string database;
        /// <summary>
        /// Default constructor
        /// </summary>
        public DbRulesAuthorizationProviderData()
        {
        }

        /// <summary>
        /// Constructor for DbRulesAuthorizationProviderData. Use this constructor when you know only
        /// the name of the DbRulesAuthorizationProvider you are creating
        /// </summary>
        /// <param name="name">Name of <see cref="DbRulesAuthorizationProvider"></see> found in configuration</param>
        public DbRulesAuthorizationProviderData(string name)
            : this(name, string.Empty)
        {
        }

        /// <summary>
        /// Constructor for DbRulesAuthorizationProviderData. Use this contstructor when you know
        /// both the name of the DBRulesAuthorizationProvider you are creating and the database instance
        /// it will use to query the database
        /// </summary>
        /// <param name="name">Name of <see cref="DbRulesAuthorizationProvider"></see> found in configuration</param>
        /// <param name="database">The name of the database instance to use</param>
        public DbRulesAuthorizationProviderData(string name, string database)
         
        {
            this.database = database;
        }

        /// <summary>
        /// Gets or sets the configured Database instance name.
        /// </summary>
        /// <value>The configured database instance.</value>
        [ConfigurationProperty("database")]
        public string Database
        {
            get { return this.database; }
            set { this.database = value; }
        }



        ///// <summary>
        ///// <para>Gets or sets the <see cref="System.Type"/> name of the provider.</para>
        ///// </summary>
        ///// <value>
        ///// <para>The type name of the provider. The default is an empty string.</para>
        ///// </value>
        //[XmlIgnore]
        //public override string TypeName
        //{
        //    get { return typeof(DbRulesAuthorizationProvider).AssemblyQualifiedName; }
        //    set
        //    {
        //    }
        //}

    }
    /// <summary>
    /// This type supports the Enterprise Library infrastructure and is not intended to be used directly from your code.
    /// Represents the process to build for an <see cref="AzManAuthorizationProvider"/> described by a <see cref="AzManAuthorizationProviderData"/> configuration object.
    /// </summary>
    /// <remarks>This type is linked to the <see cref="AzManAuthorizationProviderData"/> type and it is used by the <see cref="AuthorizationProviderCustomFactory"/> 
    /// to build the specific <see cref="IAuthorizationProvider"/> object represented by the configuration object.
    /// </remarks>
    public class DbAuthorizationProviderAssembler : IAssembler<IAuthorizationProvider, AuthorizationProviderData>
    {
        /// <summary>
        /// This method supports the Enterprise Library infrastructure and is not intended to be used directly from your code.
        /// Builds an <see cref="AzManAuthorizationProvider"/> based on an instance of <see cref="AzManAuthorizationProviderData"/>.
        /// </summary>
        /// <seealso cref="AuthorizationProviderCustomFactory"/>
        /// <param name="context">The <see cref="IBuilderContext"/> that represents the current building process.</param>
        /// <param name="objectConfiguration">The configuration object that describes the object to build. Must be an instance of <see cref="AzManAuthorizationProviderData"/>.</param>
        /// <param name="configurationSource">The source for configuration objects.</param>
        /// <param name="reflectionCache">The cache to use retrieving reflection information.</param>
        /// <returns>A fully initialized instance of <see cref="AzManAuthorizationProvider"/>.</returns>
        public IAuthorizationProvider Assemble(IBuilderContext context, AuthorizationProviderData objectConfiguration, IConfigurationSource configurationSource, ConfigurationReflectionCache reflectionCache)
        {
            DbRulesAuthorizationProviderData castedObjectConfiguration
                = (DbRulesAuthorizationProviderData)objectConfiguration;

            IAuthorizationProvider createdObject
                = new DbRulesAuthorizationProvider( );

            return createdObject;
        }
    }
}
