using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security;
using Microsoft.Practices.EnterpriseLibrary.Security.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation;
using Microsoft.Practices.EnterpriseLibrary.Security.Instrumentation;
using System.Collections.Specialized;
using System.Security.Principal;
using System.Security;
using System.Diagnostics;
namespace Kreeg.EnterpriseLibrary.Security.Database.Authorization
{
    [ConfigurationElementType(typeof(CustomAuthorizationProviderData))]
    public class DbRulesAuthorizationProvider : AuthorizationProvider 
    {

        private IDictionary<string, IAuthorizationRule> authorizationRules;
  
  
        private string database;
        private DbRulesManager mgr = null;
        public DbRulesAuthorizationProvider(NameValueCollection configurationItems)
        {
            
        }
        public DbRulesAuthorizationProvider()
        {

        }

        public DbRulesAuthorizationProvider(IDictionary<string, IAuthorizationRule> authorizationRules)
        {
            if (authorizationRules == null) throw new ArgumentNullException("authorizationRules");
           
            this.authorizationRules = authorizationRules;
        }
        public override bool Authorize(IPrincipal principal, string context)
        {
            if (principal == null)
            {
                throw new ArgumentNullException("principal");
            }
            if (context == null || context.Length == 0)
            {
                throw new ArgumentNullException("context");
            }

            if (mgr == null)
            {
                mgr = new DbRulesManager(database); //new DbRulesManager(database, this.securityConfigurationView.ConfigurationContext);
            }

           InstrumentationProvider.FireAuthorizationCheckPerformed(principal.Identity.Name, context);

            BooleanExpression expression = GetParsedExpression(context, mgr);
            if (expression == null)
            {
                throw new SecurityException(String.Format("Authorization Rule {0} not found in the database.", context));
            }

            bool result = expression.Evaluate(principal);

            if (result == false)
            {
                InstrumentationProvider.FireAuthorizationCheckFailed(principal.Identity.Name, context);
            }
            return result;
        }
        /// <summary>
        /// Retrieves a rule from the database and parses it into a boolean expresssion
        /// </summary>
        /// <param name="context">The Rule name</param>
        /// <param name="mgr">The Database Rules Manager that queries the database</param>
        /// <returns>A BooleanExpression object</returns>
        private BooleanExpression GetParsedExpression(string context, DbRulesManager mgr)
        {

            AuthorizationRuleData rule = mgr.GetRule(context);
            if (rule == null)
            {
                return null;
            }

            Parser p = new Parser();
            return p.Parse(rule.Expression);

        }
        private BooleanExpression GetParsedExpression(string ruleName)
        {
            IAuthorizationRule rule = null;
            authorizationRules.TryGetValue(ruleName, out rule);
            if (rule == null) return null;

            string expression = rule.Expression;
            Parser parser = new Parser();

            return parser.Parse(expression);
        }

    }

}
