using System;
//using Microsoft.Practices.EnterpriseLibrary.Security.Authorization;
using Microsoft.Practices.EnterpriseLibrary.Security.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using System.Data;
using System.Data.Common;
using System.Data.SqlClient;
using System.Web.Configuration;
namespace Kreeg.EnterpriseLibrary.Security.Database.Authorization
{
    /// <summary>
    /// Class for retrieving rules from the database
    /// </summary>
    public class DbRulesManager
    {

        private Microsoft.Practices.EnterpriseLibrary.Data.Database dbRules = null;
        /// <summary>
        /// Creates a Database Rules Manager instance
        /// </summary>
        /// <param name="databaseService">The Database Instance to use to query the data</param>
        /// <param name="config">The configuration context</param>
        //public DbRulesManager(string databaseService, IConfigurationSource config)
        public DbRulesManager(string databaseService)
        {
            //Database db = 
            //DatabaseProviderFactory factory = new DatabaseProviderFactory(config);
            dbRules = DatabaseFactory.CreateDatabase();
        }


        /// <summary>
        /// Retrieves a rule from the database
        /// </summary>
        /// <param name="Name">The name of the rule</param>
        /// <returns>An AuthorizationRuleData object</returns>
        public AuthorizationRuleData GetRule(string name)
        {

            AuthorizationRuleData rule = null;

            DbCommand cmd = dbRules.GetStoredProcCommand("dbo.GetRuleByName");
            dbRules.AddInParameter(cmd, "Name", DbType.AnsiString, name);

            using (IDataReader reader = dbRules.ExecuteReader(cmd))
            {
                if (reader.Read())
                {
                    rule = GetRuleFromReader(reader);
                }
            }

            return rule;
        }

        private AuthorizationRuleData GetRuleFromReader(IDataReader reader)
        {
            AuthorizationRuleData rule = new AuthorizationRuleData();
            rule.Name = reader.GetString(reader.GetOrdinal("Name"));
            rule.Expression = reader.GetString(reader.GetOrdinal("Expression"));
            
            return rule;
        }


        /// <summary>
        /// Retrieves all rules in the database as a DataSet
        /// </summary>
        /// <returns>A DataSet containing all of the rules</returns>
        public DataSet GetAllRules()
        {
            //AuthorizationRuleDataCollection rules = new AuthorizationRuleDataCollection();

            DbCommand cmd = dbRules.GetStoredProcCommand("dbo.GetAllRules");

            using (DataSet ds = dbRules.ExecuteDataSet(cmd))
            {
                return ds;
            }
        }


        /// <summary>
        /// Retrieves all rules in the database as a Collection
        /// </summary>
        /// <returns>An AuthorizationRuleDataCollection containing all of the rules</returns>
        public NamedElementCollection<AuthorizationRuleData> GetAllRulesAsCollection()
        {
            NamedElementCollection<AuthorizationRuleData> rules = new NamedElementCollection<AuthorizationRuleData>();

            DbCommand cmd = dbRules.GetStoredProcCommand("dbo.GetAllRules");

            using (IDataReader reader = dbRules.ExecuteReader(cmd))
            {
                while (reader.Read())
                {
                    AuthorizationRuleData rule = GetRuleFromReader(reader);
                    rules.Add(rule);
                }
            }
            return rules;
        }

        /// <summary>
        /// Inserts a rule into the database
        /// </summary>
        /// <param name="name">The name of the rule</param>
        /// <param name="expression">The expression defining the rule</param>
        public void InsertRule(string name, string expression)
        {
            DbCommand cmd = dbRules.GetStoredProcCommand("dbo.InsertRule");
            dbRules.AddInParameter(cmd, "Name", DbType.AnsiString, name);
            dbRules.AddInParameter(cmd, "Expression", DbType.AnsiString, expression);

            dbRules.ExecuteNonQuery(cmd);
        }

        /// <summary>
        /// Saves the rule to the database
        /// </summary>
        /// <param name="ruleId">The Rule Id</param>
        /// <param name="name">The name of the rule</param>
        /// <param name="expression">The expression</param>
        public void UpdateRuleById(int ruleId, string name, string expression)
        {
            DbCommand cmd = dbRules.GetStoredProcCommand("dbo.UpdateRuleById");
            dbRules.AddInParameter(cmd,"id", DbType.Int32, ruleId);
            dbRules.AddInParameter(cmd, "Name", DbType.AnsiString, name);
            dbRules.AddInParameter(cmd, "Expression", DbType.AnsiString, expression);

            dbRules.ExecuteNonQuery(cmd);
        }

        /// <summary>
        /// Removes a rule from the database
        /// </summary>
        /// <param name="ruleId">The ruleid to remove</param>
        public void DeleteRuleById(int ruleId)
        {
            DbCommand cmd = dbRules.GetStoredProcCommand("dbo.DeleteRuleById");
            dbRules.AddInParameter(cmd, "id", DbType.Int32, ruleId);

            dbRules.ExecuteNonQuery(cmd);
        }




    }
}
